@@ -162,6 +162,37 @@ bitarray_dealloc(bitarrayobject *self)
     Py_TYPE(self)->tp_free((PyObject *) self);
 }
 
+/* reverse n bytes (starting at start) in buffer */
+static void
+bytereverse(bitarrayobject *self, Py_ssize_t start, Py_ssize_t n)
+{
+    static char trans[256];
+    static int setup = 0;
+    Py_ssize_t i, stop = start + n;
+
+    assert(n >= 0);
+    assert(0 <= start && start <= Py_SIZE(self));
+    assert(0 <= stop && stop <= Py_SIZE(self));
+
+    if (!setup) {
+        /* setup translation table, which maps each byte to it's reversed:
+           trans = {0, 128, 64, 192, 32, 160, ..., 255} */
+        int j, k;
+
+        for (k = 0; k < 256; k++) {
+            trans[k] = 0x00;
+            for (j = 0; j < 8; j++)
+                if (1 << (7 - j) & k)
+                    trans[k] |= 1 << j;
+        }
+        setup = 1;
+    }
+
+    setunused(self);
+    for (i = start; i < stop; i++)
+        self->ob_item[i] = trans[(unsigned char) self->ob_item[i]];
+}
+
 /* copy n bits from other (starting at b) onto self (starting at a) */
 static void
 copy_n(bitarrayobject *self, Py_ssize_t a,
@@ -179,20 +210,21 @@ copy_n(bitarrayobject *self, Py_ssize_t a,
        bytes using memmove, and copy the remaining few bits individually.
        Note that the order of these two operations matters when copying
        self to self. */
-    if (self->endian == other->endian && a % 8 == 0 && b % 8 == 0 && n >= 8)
+    if (a % 8 == 0 && b % 8 == 0 && n >= 8)
     {
         const size_t bytes = n / 8;
         const Py_ssize_t bits = BITS(bytes);
 
-        assert(bits <= n && n < bits + 8);
-        if (a <= b)
-            memmove(self->ob_item + a / 8, other->ob_item + b / 8, bytes);
-
-        if (n != bits)
+        assert(bytes > 0 && bits <= n && n < bits + 8);
+        if (a > b)
             copy_n(self, bits + a, other, bits + b, n - bits);
 
-        if (a > b)
-            memmove(self->ob_item + a / 8, other->ob_item + b / 8, bytes);
+        memmove(self->ob_item + a / 8, other->ob_item + b / 8, bytes);
+        if (self->endian != other->endian)
+            bytereverse(self, a / 8, bytes);
+
+        if (a <= b)
+            copy_n(self, bits + a, other, bits + b, n - bits);
 
         return;
     }
@@ -1046,28 +1078,7 @@ When the optional `index` is given, only invert the single bit at index.");
 static PyObject *
 bitarray_bytereverse(bitarrayobject *self)
 {
-    static char trans[256];
-    static int setup = 0;
-    Py_ssize_t i;
-
-    if (!setup) {
-        /* setup translation table, which maps each byte to it's reversed:
-           trans = {0, 128, 64, 192, 32, 160, ..., 255} */
-        int j, k;
-
-        for (k = 0; k < 256; k++) {
-            trans[k] = 0x00;
-            for (j = 0; j < 8; j++)
-                if (1 << (7 - j) & k)
-                    trans[k] |= 1 << j;
-        }
-        setup = 1;
-    }
-
-    setunused(self);
-    for (i = 0; i < Py_SIZE(self); i++)
-        self->ob_item[i] = trans[(unsigned char) self->ob_item[i]];
-
+    bytereverse(self, 0, Py_SIZE(self));
     Py_RETURN_NONE;
 }
 
@@ -1866,25 +1877,13 @@ enum op_type {
 
 /* perform bitwise in-place operation */
 static int
-bitwise(bitarrayobject *self, PyObject *arg, enum op_type oper)
+bitwise(bitarrayobject *self, bitarrayobject *other, enum op_type oper)
 {
     const Py_ssize_t nbytes = Py_SIZE(self);
-    bitarrayobject *other;
     Py_ssize_t i;
 
-    if (!bitarray_Check(arg)) {
-        PyErr_SetString(PyExc_TypeError,
-                        "bitarray expected for bitwise operation");
-        return -1;
-    }
-    other = (bitarrayobject *) arg;
-    if (self->nbits != other->nbits || self->endian != other->endian) {
-        PyErr_SetString(PyExc_ValueError,
-               "bitarrays of equal length and endianness expected");
-        return -1;
-    }
-    setunused(self);
-    setunused(other);
+    assert(self->nbits == other->nbits);
+    assert(self->endian == other->endian);
     switch (oper) {
     case OP_and:
         for (i = 0; i < nbytes; i++)
@@ -1904,40 +1903,73 @@ bitwise(bitarrayobject *self, PyObject *arg, enum op_type oper)
     return 0;
 }
 
-#define BITWISE_FUNC(oper)  \
+/* return 0 if both a and b are bitarray objects, -1 on error */
+static int
+bitwise_check(PyObject *a, PyObject *b, const char *ostr)
+{
+    if (!bitarray_Check(a) || !bitarray_Check(b)) {
+        PyErr_Format(PyExc_TypeError,
+                     "unsupported operand type(s) for %s: '%s' and '%s'",
+                     ostr, Py_TYPE(a)->tp_name, Py_TYPE(b)->tp_name);
+        return -1;
+    }
+#define aa  ((bitarrayobject *) a)
+#define bb  ((bitarrayobject *) b)
+    if (aa->nbits != bb->nbits) {
+        PyErr_Format(PyExc_ValueError,
+                     "bitarrays of equal length expected for '%s'", ostr);
+        return -1;
+    }
+    if (aa->endian != bb->endian) {
+        PyErr_Format(PyExc_ValueError,
+                     "bitarrays of equal endianness expected for '%s'", ostr);
+        return -1;
+    }
+#undef aa
+#undef bb
+    return 0;
+}
+
+#define BITWISE_FUNC(oper, ostr)                                    \
 static PyObject *                                                   \
-bitarray_ ## oper (bitarrayobject *self, PyObject *other)           \
+bitarray_ ## oper (PyObject *self, PyObject *other)                 \
 {                                                                   \
     PyObject *res;                                                  \
                                                                     \
-    res = bitarray_copy(self);                                      \
+    if (bitwise_check(self, other, ostr) < 0)                       \
+        return NULL;                                                \
+    res = bitarray_copy((bitarrayobject *) self);                   \
     if (res == NULL)                                                \
         return NULL;                                                \
-    if (bitwise((bitarrayobject *) res, other, OP_ ## oper) < 0) {  \
+    if (bitwise((bitarrayobject *) res,                             \
+                (bitarrayobject *) other, OP_ ## oper) < 0) {       \
         Py_DECREF(res);                                             \
         return NULL;                                                \
     }                                                               \
     return res;                                                     \
 }
 
-BITWISE_FUNC(and)               /* bitarray_and */
-BITWISE_FUNC(or)                /* bitarray_or  */
-BITWISE_FUNC(xor)               /* bitarray_xor */
+BITWISE_FUNC(and, "&")               /* bitarray_and */
+BITWISE_FUNC(or,  "|")               /* bitarray_or  */
+BITWISE_FUNC(xor, "^")               /* bitarray_xor */
 
 
-#define BITWISE_IFUNC(oper)  \
+#define BITWISE_IFUNC(oper, ostr)                            \
 static PyObject *                                            \
-bitarray_i ## oper (bitarrayobject *self, PyObject *other)   \
+bitarray_i ## oper (PyObject *self, PyObject *other)         \
 {                                                            \
-    if (bitwise(self, other, OP_ ## oper) < 0)               \
+    if (bitwise_check(self, other, ostr) < 0)                \
+        return NULL;                                         \
+    if (bitwise((bitarrayobject *) self,                     \
+                (bitarrayobject *) other, OP_ ## oper) < 0)  \
         return NULL;                                         \
     Py_INCREF(self);                                         \
     return (PyObject *) self;                                \
 }
 
-BITWISE_IFUNC(and)              /* bitarray_iand */
-BITWISE_IFUNC(or)               /* bitarray_ior  */
-BITWISE_IFUNC(xor)              /* bitarray_ixor */
+BITWISE_IFUNC(and, "&=")             /* bitarray_iand */
+BITWISE_IFUNC(or,  "|=")             /* bitarray_ior  */
+BITWISE_IFUNC(xor, "^=")             /* bitarray_ixor */
 
 
 static PyNumberMethods bitarray_as_number = {
