@@ -11,7 +11,6 @@
 #include "Python.h"
 #include "bitarray.h"
 
-#define HEXDIGITS  "0123456789abcdef"
 #define IS_LE(a)  ((a)->endian == ENDIAN_LITTLE)
 #define IS_BE(a)  ((a)->endian == ENDIAN_BIG)
 
@@ -37,7 +36,7 @@ ensure_bitarray(PyObject *obj)
     return 0;
 }
 
-/************* start of actual functionality in this module ***************/
+/* ------------------------------- count_n ----------------------------- */
 
 /* return the smallest index i for which a.count(1, 0, i) == n, or when
    n exceeds the total count return -1  */
@@ -92,6 +91,43 @@ count_to_n(bitarrayobject *a, Py_ssize_t n)
     return i;
 }
 
+static PyObject *
+count_n(PyObject *module, PyObject *args)
+{
+    PyObject *a;
+    Py_ssize_t n, i;
+
+    if (!PyArg_ParseTuple(args, "On:count_n", &a, &n))
+        return NULL;
+    if (ensure_bitarray(a) < 0)
+        return NULL;
+
+    if (n < 0) {
+        PyErr_SetString(PyExc_ValueError, "non-negative integer expected");
+        return NULL;
+    }
+#define aa  ((bitarrayobject *) a)
+    if (n > aa->nbits)  {
+        PyErr_SetString(PyExc_ValueError, "n larger than bitarray size");
+        return NULL;
+    }
+    i = count_to_n(aa, n);        /* do actual work here */
+#undef aa
+    if (i < 0) {
+        PyErr_SetString(PyExc_ValueError, "n exceeds total count");
+        return NULL;
+    }
+    return PyLong_FromSsize_t(i);
+}
+
+PyDoc_STRVAR(count_n_doc,
+"count_n(a, n, /) -> int\n\
+\n\
+Return the smallest index `i` for which `a[:i].count() == n`.\n\
+Raises `ValueError`, when n exceeds total count (`a.count()`).");
+
+/* ----------------------------- right index --------------------------- */
+
 /* return index of last occurrence of vi, -1 when x is not in found. */
 static Py_ssize_t
 find_last(bitarrayobject *a, int vi)
@@ -131,56 +167,6 @@ find_last(bitarrayobject *a, int vi)
     return -1;
 }
 
-static int
-hex_to_int(char c)
-{
-    if ('0' <= c && c <= '9')
-        return c - '0';
-    if ('a' <= c && c <= 'f')
-        return c - 'a' + 10;
-    if ('A' <= c && c <= 'F')
-        return c - 'A' + 10;
-    return -1;
-}
-
-/****************************** Module functions **************************/
-
-static PyObject *
-count_n(PyObject *module, PyObject *args)
-{
-    PyObject *a;
-    Py_ssize_t n, i;
-
-    if (!PyArg_ParseTuple(args, "On:count_n", &a, &n))
-        return NULL;
-    if (ensure_bitarray(a) < 0)
-        return NULL;
-
-    if (n < 0) {
-        PyErr_SetString(PyExc_ValueError, "non-negative integer expected");
-        return NULL;
-    }
-#define aa  ((bitarrayobject *) a)
-    if (n > aa->nbits)  {
-        PyErr_SetString(PyExc_ValueError, "n larger than bitarray size");
-        return NULL;
-    }
-    i = count_to_n(aa, n);        /* do actual work here */
-#undef aa
-    if (i < 0) {
-        PyErr_SetString(PyExc_ValueError, "n exceeds total count");
-        return NULL;
-    }
-    return PyLong_FromSsize_t(i);
-}
-
-PyDoc_STRVAR(count_n_doc,
-"count_n(a, n, /) -> int\n\
-\n\
-Return the smallest index `i` for which `a[:i].count() == n`.\n\
-Raises `ValueError`, when n exceeds total count (`a.count()`).");
-
-
 static PyObject *
 r_index(PyObject *module, PyObject *args)
 {
@@ -210,6 +196,34 @@ PyDoc_STRVAR(rindex_doc,
 Return the rightmost index of `bool(value)` in bitarray.\n\
 Raises `ValueError` if the value is not present.");
 
+/* --------------------------- unary functions ------------------------- */
+
+static PyObject *
+parity(PyObject *module, PyObject *a)
+{
+    Py_ssize_t i, nbytes;
+    unsigned char par = 0;
+
+    if (ensure_bitarray(a) < 0)
+        return NULL;
+
+    nbytes = Py_SIZE(a);
+#define aa  ((bitarrayobject *) a)
+    setunused(aa);
+    for (i = 0; i < nbytes; i++)
+        par ^= aa->ob_item[i];
+#undef aa
+
+    return PyBool_FromLong((long) bitcount_lookup[par] % 2);
+}
+
+PyDoc_STRVAR(parity_doc,
+"parity(a, /) -> bool\n\
+\n\
+Return the parity of bitarray `a`.  This is equivalent\n\
+to `bool(a.count() % 2)` (but more efficient).");
+
+/* --------------------------- binary functions ------------------------ */
 
 enum kernel_type {
     KERN_cand,     /* count bitwise and -> int */
@@ -219,7 +233,7 @@ enum kernel_type {
 };
 
 static PyObject *
-two_bitarray_func(PyObject *args, enum kernel_type kern, char *format)
+two_bitarray_func(PyObject *args, enum kernel_type kern, const char *format)
 {
     Py_ssize_t res = 0, nbytes, i;
     PyObject *a, *b;
@@ -232,9 +246,14 @@ two_bitarray_func(PyObject *args, enum kernel_type kern, char *format)
 
 #define aa  ((bitarrayobject *) a)
 #define bb  ((bitarrayobject *) b)
-    if (aa->nbits != bb->nbits || aa->endian != bb->endian) {
+    if (aa->nbits != bb->nbits) {
         PyErr_SetString(PyExc_ValueError,
-                        "bitarrays of equal length and endianness expected");
+                        "bitarrays of equal length expected");
+        return NULL;
+    }
+    if (aa->endian != bb->endian) {
+        PyErr_SetString(PyExc_ValueError,
+                        "bitarrays of equal endianness expected");
         return NULL;
     }
     setunused(aa);
@@ -306,32 +325,7 @@ Return True if bitarray `a` is a subset of bitarray `b` (False otherwise).\n\
 efficient since we can stop as soon as one mismatch is found, and no\n\
 intermediate bitarray object gets created.");
 
-
-static PyObject *
-parity(PyObject *module, PyObject *a)
-{
-    Py_ssize_t i, nbytes;
-    unsigned char par = 0;
-
-    if (ensure_bitarray(a) < 0)
-        return NULL;
-
-    nbytes = Py_SIZE(a);
-#define aa  ((bitarrayobject *) a)
-    setunused(aa);
-    for (i = 0; i < nbytes; i++)
-        par ^= aa->ob_item[i];
-#undef aa
-
-    return PyBool_FromLong((long) bitcount_lookup[par] % 2);
-}
-
-PyDoc_STRVAR(parity_doc,
-"parity(a, /) -> bool\n\
-\n\
-Return the parity of bitarray `a`.  This is equivalent\n\
-to `bool(a.count() % 2)` (but more efficient).");
-
+/* ---------------------------- serialization -------------------------- */
 
 static PyObject *
 serialize(PyObject *module, PyObject *a)
@@ -365,6 +359,21 @@ Return a serialized representation of the bitarray, which may be passed to\n\
 `deserialize()`.  It efficiently represents the bitarray object (including\n\
 its endianness) and is guaranteed not to change in future releases.");
 
+/* ----------------------------- hexadecimal --------------------------- */
+
+static const char hexdigits[] = "0123456789abcdef";
+
+static int
+hex_to_int(char c)
+{
+    if ('0' <= c && c <= '9')
+        return c - '0';
+    if ('a' <= c && c <= 'f')
+        return c - 'a' + 10;
+    if ('A' <= c && c <= 'F')
+        return c - 'A' + 10;
+    return -1;
+}
 
 static PyObject *
 ba2hex(PyObject *module, PyObject *a)
@@ -383,6 +392,7 @@ ba2hex(PyObject *module, PyObject *a)
         return NULL;
     }
 
+    /* strsize = aa->nbits / 4;  could make strsize odd */
     strsize = 2 * Py_SIZE(a);
     str = (char *) PyMem_Malloc(strsize);
     if (str == NULL)
@@ -392,8 +402,8 @@ ba2hex(PyObject *module, PyObject *a)
     be = IS_BE(aa);
     for (i = 0; i < strsize; i += 2) {
         unsigned char c = aa->ob_item[i / 2];
-        str[i + le] = HEXDIGITS[c >> 4];
-        str[i + be] = HEXDIGITS[0x0f & c];
+        str[i + le] = hexdigits[c >> 4];
+        str[i + be] = hexdigits[0x0f & c];
     }
     result = Py_BuildValue("s#", str, aa->nbits / 4);
 #undef aa
@@ -404,26 +414,21 @@ ba2hex(PyObject *module, PyObject *a)
 PyDoc_STRVAR(ba2hex_doc,
 "ba2hex(bitarray, /) -> hexstr\n\
 \n\
-Return a string containing with hexadecimal representation of\n\
+Return a string containing the hexadecimal representation of\n\
 the bitarray (which has to be multiple of 4 in length).");
 
 
-/* translate hexadecimal digits into the bitarray's buffer */
+/* Translate hexadecimal digits into the bitarray's buffer.
+   Each digit corresponds to 4 bits in the bitarray.
+   The number of digits may be odd. */
 static PyObject *
 hex2ba(PyObject *module, PyObject *args)
 {
     PyObject *a;
     char *str;
     Py_ssize_t i, strsize;
-    int le, be;
-    static char hex2int[256];
-    static int setup = 0;
+    int le, be, x, y;
 
-    if (!setup) {
-        for (i = 0; i < 256; i++)
-            hex2int[i] = hex_to_int((char) i);
-        setup = 1;
-    }
     if (!PyArg_ParseTuple(args, "Os#", &a, &str, &strsize))
         return NULL;
     if (ensure_bitarray(a) < 0)
@@ -439,8 +444,8 @@ hex2ba(PyObject *module, PyObject *args)
     be = IS_BE(aa);
     assert(le + be == 1 && str[strsize] == 0);
     for (i = 0; i < strsize; i += 2) {
-        char x = hex2int[(unsigned char) str[i + le]];
-        char y = hex2int[(unsigned char) str[i + be]];
+        x = hex_to_int(str[i + le]);
+        y = hex_to_int(str[i + be]);
         if (x < 0 || y < 0) {
             /* ignore the terminating NUL - happends when strsize is odd */
             if (i + le == strsize) /* str[i+le] is NUL */
@@ -461,6 +466,8 @@ hex2ba(PyObject *module, PyObject *args)
     Py_RETURN_NONE;
 }
 
+/* --------------------------------------------------------------------- */
+
 /* set bitarray_type_obj (bato) */
 static PyObject *
 set_bato(PyObject *module, PyObject *obj)
@@ -472,11 +479,11 @@ set_bato(PyObject *module, PyObject *obj)
 static PyMethodDef module_functions[] = {
     {"count_n",   (PyCFunction) count_n,   METH_VARARGS, count_n_doc},
     {"rindex",    (PyCFunction) r_index,   METH_VARARGS, rindex_doc},
+    {"parity",    (PyCFunction) parity,    METH_O,       parity_doc},
     {"count_and", (PyCFunction) count_and, METH_VARARGS, count_and_doc},
     {"count_or",  (PyCFunction) count_or,  METH_VARARGS, count_or_doc},
     {"count_xor", (PyCFunction) count_xor, METH_VARARGS, count_xor_doc},
     {"subset",    (PyCFunction) subset,    METH_VARARGS, subset_doc},
-    {"parity",    (PyCFunction) parity,    METH_O,       parity_doc},
     {"serialize", (PyCFunction) serialize, METH_O,       serialize_doc},
     {"ba2hex",    (PyCFunction) ba2hex,    METH_O,       ba2hex_doc},
     {"_hex2ba",   (PyCFunction) hex2ba,    METH_VARARGS, 0},
